import json
import logging
import threading

import os
import re
import sys
import random
import time
import boto3

from botocore.exceptions import ClientError
_rand = str(random.randint(10, 2400))
#
s3client = boto3.client('s3')
iam_client = boto3.client('iam')


logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)


def assume_role(account):
    assumedRoleObject = {}
    sts_client = boto3.client('sts')
    role_arn = 'arn:aws:iam::' + \
        account['AccountId'] + ':role/' + \
        account['adminRole']
    try:
        assumedRoleObject = sts_client.assume_role(
            RoleArn=role_arn,
            RoleSessionName="NewAccountRole"
        )
    except Exception as e:
        logger.warning(e)
        assumedRoleObject['status'] = str(e)

    return assumedRoleObject


def deploy_cloudformation(event):
    logger.debug("Deploying Resources")
    _rand = str(random.randint(10, 24400))
    dynamodbkey = event['dynamodbkey']
    tname = ''
    stname = event['TemplateURL'].split('?')[0].split('/')
    for _stack in stname:
        if _stack.find('.json') > -1:
            tname = _stack
        if _stack.find('.yaml') > -1:
            tname = _stack
    event['tname'] = tname
    sstname = tname.split('.')[0].replace('_', '')
    stack_name = f"scenhactfact{_rand}-{sstname}"
    stack_region = event['stack_region']
    TemplateURL = event['TemplateURL']
    taraccount = event['taraccount']

    datestamp = time.strftime("%d/%m/%Y")
    #
    targetAcct = {'AccountId': taraccount, 'adminRole': 'AWSControlTowerExecution'}
    acct_b = assume_role(targetAcct)
    # print(acct_b)
    # return stack_name

    client = boto3.client('cloudformation',
                          aws_access_key_id=acct_b['Credentials']['AccessKeyId'],
                          aws_secret_access_key=acct_b['Credentials']['SecretAccessKey'],
                          aws_session_token=acct_b['Credentials']['SessionToken'],
                          region_name=stack_region)

    logger.info("Creating stack " + stack_name + " in " + taraccount)
    logger.info("Creating stack TemplateURL[" + TemplateURL + "]")

    try:
        creating_stack = False
        create_stack_response = client.create_stack(
            StackName=stack_name,
            TemplateURL=TemplateURL,
            Parameters=[],
            NotificationARNs=[],
            Capabilities=[
                'CAPABILITY_NAMED_IAM'
            ],
            OnFailure='ROLLBACK',
            Tags=[
                {
                    'Key': 'EnhancedAcctFact',
                    'Value': stack_name
                },
                {
                    'Key': 'DeployDate',
                    'Value': datestamp
                },
                {
                    'Key': 'Dynamodbkey',
                    'Value': dynamodbkey
                }
            ]
        )

    except ClientError as e:
        to_raise = ['AlreadyExistsException', 'LimitExceededException',
                    'InsufficientCapabilitiesException']
        logger.error(e.response)
        if e.response['Error']['Code'] in to_raise:
            client.delete_stack(StackName=stack_name)
            logger.error(e)
            raise
        creating_stack = True
        logger.warning(e)
        logger.warning("Retrying...")
        time.sleep(10)
    except Exception as e:
        logger.error(e)
    time.sleep(3)
    logger.info("Stack creation in process...")
    event_list = client.describe_stack_events(StackName=stack_name).get("StackEvents")
    stack_event = event_list[0]
    if stack_event.get('ResourceType') == 'AWS::CloudFormation::Stack':
        stack_status = stack_event.get('ResourceStatus')

    event['stack_name'] = stack_name
    event['stack_status'] = stack_status
    return event


def check_status_cloudformation(event):

    logger.debug("Checking Resources")
    stack_name = event['stack_name']
    stack_region = event['stack_region']
    TemplateURL = event['TemplateURL']
    taraccount = event['taraccount']
    stack_status = ''
    # Get target account info
    targetAcct = {'AccountId': taraccount, 'adminRole': 'AWSControlTowerExecution'}
    acct_b = assume_role(targetAcct)

    client = boto3.client('cloudformation',
                          aws_access_key_id=acct_b['Credentials']['AccessKeyId'],
                          aws_secret_access_key=acct_b['Credentials']['SecretAccessKey'],
                          aws_session_token=acct_b['Credentials']['SessionToken'],
                          region_name=stack_region)

    logger.info("Checking  stack " + stack_name + " in " + taraccount)
    logger.info("Checking stack TemplateURL[" + TemplateURL + "]")
    #
    stack_status = client.describe_stacks(StackName=stack_name)['Stacks'][0]['StackStatus']
    return stack_status


def lambda_handler(event, context):
    print(json.dumps(event))
    _rand = str(random.randint(10, 2400))
    event['TemplateURL'] = event['stackstodeploy'][event['currentstack']]

    event = deploy_cloudformation(event)
    event['currentstack'] = event['currentstack'] + 1
    if event['currentstack'] == event['numofstacks']:
        event['morestacks'] = 'No'

    return event
