import os
import json
import boto3
import random

# ====
s3client = boto3.client('s3')
sc_client = boto3.client('servicecatalog')
org_client = boto3.client('organizations')

def sc_deactivat_ver(_newversion, _prodinfo):
    for _product in _prodinfo:
        if _newversion != _product['Name']:
            response = sc_client.update_provisioning_artifact(
                AcceptLanguage='en',
                ProductId=_product['ProductId'],
                ProvisioningArtifactId=_product['ProvisioningArtifactId'],
                Active=False,
                Guidance='DEPRECATED')
    return 0


def sc_create_pa(ProductId, version, description, template):
    _rand = random.randint(323, 24200)
    IdempotencyToken = "tok%s" % _rand
    rep = sc_client.create_provisioning_artifact(AcceptLanguage='en', ProductId=ProductId, Parameters={
        'Name': version,
        'Description': description,
        'Info': {
            "LoadTemplateFromURL": template
        },
        'Type': 'CLOUD_FORMATION_TEMPLATE',
        'DisableTemplateValidation': True
    },
        IdempotencyToken=IdempotencyToken
    )
    return rep


def sc_get_prod_details(name):
    ret = []
    rret = {}
    rep = sc_client.describe_product_as_admin(
        AcceptLanguage='en', Name=name)
    try:
        rep = sc_client.describe_product_as_admin(
            AcceptLanguage='en', Name=name)
        if 'ProvisioningArtifactSummaries' in rep:
            rret['ProductId'] = rep['ProductViewDetail']['ProductViewSummary']['ProductId']
            lpaths = sc_client.list_launch_paths(
                AcceptLanguage='en', ProductId=rep['ProductViewDetail']['ProductViewSummary']['ProductId'])['LaunchPathSummaries']
            for pa in rep['ProvisioningArtifactSummaries']:
                resp2 = sc_client.describe_provisioning_artifact(
                    AcceptLanguage='en', ProvisioningArtifactId=pa['Id'], ProductId=rep['ProductViewDetail']['ProductViewSummary']['ProductId'], Verbose=True)
                rret['template'] = resp2['Info']['CloudFormationTemplate']
                if resp2['ProvisioningArtifactDetail']['Active']:
                    dinfo = {'ProvisioningArtifactId': resp2['ProvisioningArtifactDetail']
                             ['Id'], 'Active': resp2['ProvisioningArtifactDetail']['Active']}
                    rret['ProvisioningArtifactId'] = resp2['ProvisioningArtifactDetail']['Id']

            rret['pathId'] = lpaths[0]['Id']

    except Exception as e:
        print(e)
    return rret


def sc_find_product(name):
    _ret = 'not found'
    try:
        rep = sc_client.describe_product_as_admin(
            AcceptLanguage='en', Name=name)

        _ret = 'found'
    except Exception as e:
        e = str(e)
        print(e)
        if e.find('ResourceNotFoundException') > -1:
            print(name + ' not found')
    return _ret


def sc_describe_product(name):
    rep = {}
    try:
        rep = sc_client.describe_product_as_admin(
            AcceptLanguage='en', Name=name)
    except Exception as e:
        e = str(e)
        if e.find('ResourceNotFoundException') > -1:
            print(name + ' not found')
            _ret = 'not found'
            return _ret
    ProductId = rep['ProductViewDetail']['ProductViewSummary']['ProductId']
    _ret = []
    for _prodversion in rep['ProvisioningArtifactSummaries']:
        _version = {'ProductId': ProductId, 'ProvisioningArtifactId': _prodversion[
            'Id'], 'Name': _prodversion['Name'], 'Description': _prodversion['Description']}
        _ret.append(_version)
    return _ret


def mgt_ent_sc_updateProduct(_sc_productname, template):
    _prodinfo = sc_describe_product(_sc_productname)
    _versions = len(_prodinfo) + 1
    ProductId = _prodinfo[0]['ProductId']
    version = 'Latest %s.0' % _versions
    description = 'Decription %s' % version
    pa_rep = sc_create_pa(
        ProductId, version, description, template)
    _ret = {'ProductId': ProductId, 'version': version,
            'ProvisioningArtifactId': pa_rep['ProvisioningArtifactDetail']['Id']}
    _prodinfo = sc_describe_product(_sc_productname)
    sc_deactivat_ver(version, _prodinfo)
    return _ret


def bucket_putpriv(bucket, key, body, type):
    srep = s3client.put_object(
        ACL='private', Body=body, Bucket=bucket, Key=key, ContentType=type,)
    print(srep)
    return srep


def bucket_get_obj(bucket, bkey):
    object = s3client.get_object(Bucket=bucket, Key=bkey)
    return(object['Body'].read().decode('utf-8'))


def listdirectry(bucket, prefix):
    _ret = []
    ret = s3client.list_objects_v2(
        Bucket=bucket, Delimiter=',', MaxKeys=123, Prefix=prefix)
    for key in ret['Contents']:
        keyarray = key['Key'].split('/')
        _scount = len(keyarray)
        _ret.append(keyarray[_scount - 2])
        _rret = list(set(_ret))
        _rret.sort()
    _dircount = []
    _ret2 = []
    for _dir in _rret:
        _ncount = 0
        _prefix = f'{prefix}{_dir}/'
        print(_prefix)
        _nret = s3client.list_objects_v2(
            Bucket=bucket, Delimiter=',', MaxKeys=123, Prefix=_prefix)
        if 'Contents' in _nret:
            _ncount = len(_nret['Contents'])
        _entry = f'{_dir}[{_ncount}]'
        _ret2.append(_entry)

    return _ret2


def listdirectryitems(bucket, prefix, strip):
    _ret = []
    ret = s3client.list_objects_v2(
        Bucket=bucket, Delimiter=',', MaxKeys=123, Prefix=prefix)
    if 'Contents' in ret:
        for key in ret['Contents']:
            if strip == 'yes':
                _keyarr = key['Key'].split('/')
                _klen = len(_keyarr)
                _ret.append(_keyarr[_klen - 1])
            else:
                _ret.append(key['Key'])
            _ret.sort()

    return _ret


def get_ous_fromtemplate(template):
    _tp1 = template.split("ManagedOrganizationalUnit:")[
        1].split('Resources:')[0].split(' - ')
    tpar = []
    for gval in _tp1:
        if gval.find('9') > -1:
            t = gval.replace('\n', '')
            tpar.append(t.rstrip())
    return tpar

def ListAccounts():
    _ar = []
    try:
        _ret = org_client.list_roots()
        _id = _ret['Roots'][0]['Id']
        _ret2 = org_client.list_organizational_units_for_parent(
            ParentId=_id)
        for _o in _ret2['OrganizationalUnits']:
            _accts = org_client.list_accounts_for_parent(
                ParentId=_o['Id'])
            for _act in _accts['Accounts']:
                if _o['Name'].find('Core') > -1:
                    donothigpartofcore = 1
                else:
                    _b = f"{_o['Name']}_{_act['Name']}_{_act['Id']}_{_act['Status']}"
                    _ar.append(_b)
            _ar.sort()
    except Exception as e:
        _ar.append('Error_gettingAccounts ' + str(e))
    return _ar

def lambda_handler(event, context):
    print(json.dumps(event))
    ret = {}
    bucket = os.environ['S3bucket']
    scproductname = os.environ['scproductname']
    Enhscproductname = os.environ['Enhscproductname']
    prefix = os.environ['prefix']
    #sc_product_scenhanceaf_templates_new.json
    sc_product_scenhanceafjson_key = f'{prefix}sc_product_scenhanceaf_templates_new.json'
    sc_product_scenhanceafjson_template_key = f'{prefix}sc_product_scenhanceaf_templates_new.json'
    
    sc_product_scenhanceafjson_key_new = f'{prefix}sc_product_scenhanceaf_new.json'
    sc_profjson_key_new = f'{prefix}sc_product_scenhanceaf_acftnew.json'
    configparakey = f'{prefix}src/config/enparameter.json'
    configparajson = json.loads(bucket_get_obj(
        bucket, configparakey))['ehancedparameters']
    postprefix = f'{prefix}src/postaccount/'
    post_stacks = listdirectryitems(
        bucket, postprefix, 'yes')
    post_stacks.append('None')
    _prodinfo = sc_get_prod_details(scproductname)
    pou = get_ous_fromtemplate(_prodinfo['template'])
    sc_product_json = json.loads(bucket_get_obj(
        bucket, sc_product_scenhanceafjson_key_new))

    originalpara = sc_product_json['Parameters']
    for _para in configparajson:
        pp = {_para['Name']: {'Description': _para['Description'],
                              'Type': _para['Type'], 'AllowedValues': post_stacks}}
        sc_product_json['Parameters'][_para['Name']] = {
            'Description': _para['Description'], 'Type': _para['Type'], 'AllowedValues': post_stacks, 'Default': 'None'}
        sc_product_json['Resources']['CallStepFunction']['Properties'][_para['Name']] = {
            "Ref": _para['Name']}
    bucket_putpriv(bucket, sc_product_scenhanceafjson_key_new, json.dumps(
        sc_product_json, indent=4), 'text/json')
    bucket_putpriv(bucket, sc_profjson_key_new,
                   _prodinfo['template'], 'text/json')

    loadtemplatefromurl = f'https://{bucket}.s3.amazonaws.com/{sc_product_scenhanceafjson_key_new}'
    mgt_ent_sc_updateProduct(
        Enhscproductname, loadtemplatefromurl)
    
    #template only
    targetAccounts = ListAccounts()
    sc_product_jsonTeplate = json.loads(bucket_get_obj(
        bucket, sc_product_scenhanceafjson_template_key))
    originalparat = sc_product_jsonTeplate['Parameters']
    sc_product_jsonTeplate['Parameters']['TargetAccount']['AllowedValues'] = targetAccounts
    for _para in configparajson:
        pp = {_para['Name']: {'Description': _para['Description'],
                              'Type': _para['Type'], 'AllowedValues': post_stacks}}
        sc_product_jsonTeplate['Parameters'][_para['Name']] = {
            'Description': _para['Description'], 'Type': _para['Type'], 'AllowedValues': post_stacks, 'Default': 'None'}
        sc_product_jsonTeplate['Resources']['CallStepFunction']['Properties'][_para['Name']] = {
            "Ref": _para['Name']}
    
    bucket_putpriv(bucket, sc_product_scenhanceafjson_template_key,json.dumps(
        sc_product_jsonTeplate, indent=4), 'text/json')
                   
    loadtemplatefromurl_template = f'https://{bucket}.s3.amazonaws.com/{sc_product_scenhanceafjson_template_key}'
    mgt_ent_sc_updateProduct(
        "AccountFactoryEnhancedTemplate", loadtemplatefromurl_template)
        

    return ret
