import json
import boto3
import os
import random
import time
import logging
import urllib3
import threading
import botocore

s3client = boto3.client('s3')
sc_client = boto3.client('servicecatalog')
http = urllib3.PoolManager()
logger = logging.getLogger()


def gen_surl(bucketname, keyname):
    url = s3client.generate_presigned_url(ClientMethod='get_object', Params={
                                          'Bucket': bucketname, 'Key': keyname})
    return url


def cfnresponse(event, context, responseStatus, responseData, physicalResourceId=None, noEcho=False):
    responseBody = {}
    responseBody['Status'] = responseStatus
    responseBody['Reason'] = 'See the details in CloudWatch Log Stream: ' + \
        context.log_stream_name
    responseBody['PhysicalResourceId'] = physicalResourceId or context.log_stream_name
    responseBody['StackId'] = event['StackId']
    responseBody['RequestId'] = event['RequestId']
    responseBody['LogicalResourceId'] = event['LogicalResourceId']
    responseBody['NoEcho'] = noEcho
    responseBody['Data'] = responseData
    json_responseBody = json.dumps(responseBody)
    headers = {'content-type': '',
               'content-length': str(len(json_responseBody))}
    try:
        response = http.request(
            'PUT', event['ResponseURL'], body=json_responseBody.encode('utf-8'), headers=headers)
        logger.debug('Status code: ' + response.reason)
    except Exception as e:
        logger.error(
            'cfnresponse(..) failed executing requests.put(..): ' + str(e))


def timeout(event, context):
    logging.error(
        'Execution is about to time out, sending failure response to CloudFormation')
    cfnresponse(event, context, 'FAILED', {})


sfn_client = boto3.client('stepfunctions')


def sfn_startex(stateMachineArn, input):
    _rand = str(random.randint(323, 24200))
    name = f"sfnscenacct{_rand}"
    resp = sfn_client.start_execution(
        stateMachineArn=stateMachineArn, name=name, input=input)
    return resp['executionArn']


def sc_descproviproduct(ppid):
    resp = sc_client.describe_provisioned_product(
        AcceptLanguage='en', Id=ppid)
    recid = resp['ProvisionedProductDetail']['LastProvisioningRecordId']

    ridresp = sc_client.describe_record(
        AcceptLanguage='en', Id=recid)
    print(ridresp)
    rec_outputs = ridresp['RecordOutputs']
    __ret = {}
    for out in rec_outputs:
        if out['OutputKey'] == "AccountId":
            __ret['accountid'] = out['OutputValue']
        if out['OutputKey'] == "AccountEmail":
            __ret['accountemail'] = out['OutputValue']

    return __ret


def lambda_handler(event, context):
    print(json.dumps(event))
    bucket = os.environ['S3bucket']
    prefix = os.environ['prefix']
    if 'ResourceProperties' in event:
        timer = threading.Timer((context.get_remaining_time_in_millis(
        ) / 1000.00) - 0.5, timeout, args=[event, context])
        timer.start()
        status = 'SUCCESS'

        timer.cancel()
        cfnresponse(event, context, status, event)
        if event['RequestType'] == 'Create':
            sevent = {}
            _stacks = []
            for _step in event['ResourceProperties']:
                if _step.find('Step') > -1:
                    if event['ResourceProperties'][_step].find('None') > -1:
                        nostackskip = 1
                    else:
                        _dastep = f"{_step}-{prefix}src/postaccount/{event['ResourceProperties'][_step]}"
                        _stackey = f"{prefix}src/postaccount/{event['ResourceProperties'][_step]}"
                        _stacksurl = gen_surl(
                            bucket, _stackey)
                        _stacks.append(_stacksurl)
            _stacks.sort()
            ppid = event['ResourceProperties']['ppid']
            ppid = event['ResourceProperties']['ppid']
            if ppid == "_templates_only":
                acctfacinfo = {
                    'accountid': event['ResourceProperties']['TargetAccount'].split('_')[2]}
            else:
                acctfacinfo = sc_descproviproduct(ppid)
            _rand = str(random.randint(323, 24200))
            sevent['dynamodbkey'] = f"acctfactenh{_rand}"

            sevent['RequestId'] = event['RequestId']
            sevent['event_action'] = 'deploystack'
            sevent['stackstodeploy'] = _stacks
            sevent['morestacks'] = 'Yes'
            sevent['numofstacks'] = len(_stacks)
            sevent['currentstack'] = 0
            sevent['stack_region'] = os.environ['AWS_REGION']
            sevent['stack_name'] = 'scenhtfac'
            sevent['workingstack'] = _stacks[0]
            sevent['taraccount'] = acctfacinfo['accountid']
            sfnarn = event['ResourceProperties']['sfnarn']
            inputs = json.dumps(sevent)
            event['sfnstatus'] = sfn_startex(sfnarn, inputs)

        if event['RequestType'] == 'Delete':

            event['event_action'] = 'Delete'
            event['status'] = 'Deleting....'

    return event