import os
import json
import boto3
import random
import botocore
# ====
import logging
import threading
import urllib3
# ===
http = urllib3.PoolManager()
s3client = boto3.client('s3')
logger = logging.getLogger()
org_client = boto3.client('organizations')

s3client = boto3.client(
    's3', 'us-east-1', config=botocore.config.Config(s3={'addressing_style': 'path'}))
sc_client = boto3.client('servicecatalog')


def cfnresponse(event, context, responseStatus, responseData, physicalResourceId=None, noEcho=False):
    responseBody = {}
    responseBody['Status'] = responseStatus
    responseBody['Reason'] = 'See the details in CloudWatch Log Stream: ' + \
        context.log_stream_name
    responseBody['PhysicalResourceId'] = physicalResourceId or context.log_stream_name
    responseBody['StackId'] = event['StackId']
    responseBody['RequestId'] = event['RequestId']
    responseBody['LogicalResourceId'] = event['LogicalResourceId']
    responseBody['NoEcho'] = noEcho
    responseBody['Data'] = responseData
    json_responseBody = json.dumps(responseBody)
    headers = {'content-type': '',
               'content-length': str(len(json_responseBody))}
    try:
        response = http.request(
            'PUT', event['ResponseURL'], body=json_responseBody.encode('utf-8'), headers=headers)
        logger.debug('Status code: ' + response.reason)
    except Exception as e:
        logger.error(
            'cfnresponse(..) failed executing requests.put(..): ' + str(e))


def timeout(event, context):
    logging.error(
        'Execution is about to time out, sending failure response to CloudFormation')
    cfnresponse(event, context, 'FAILED', {})


def sc_deactivat_ver(_newversion, _prodinfo):
    for _product in _prodinfo:
        if _newversion != _product['Name']:
            response = sc_client.update_provisioning_artifact(
                AcceptLanguage='en',
                ProductId=_product['ProductId'],
                ProvisioningArtifactId=_product['ProvisioningArtifactId'],
                Active=False,
                Guidance='DEPRECATED')
    return 0


def sc_create_pa(ProductId, version, description, template):
    _rand = random.randint(323, 24200)
    IdempotencyToken = "tok%s" % _rand
    rep = sc_client.create_provisioning_artifact(AcceptLanguage='en', ProductId=ProductId, Parameters={
        'Name': version,
        'Description': description,
        'Info': {
            "LoadTemplateFromURL": template
        },
        'Type': 'CLOUD_FORMATION_TEMPLATE',
        'DisableTemplateValidation': True
    },
        IdempotencyToken=IdempotencyToken
    )
    return rep


def sc_get_prod_details(name):
    ret = []
    rret = {}
    rep = sc_client.describe_product_as_admin(
        AcceptLanguage='en', Name=name)
    try:
        rep = sc_client.describe_product_as_admin(
            AcceptLanguage='en', Name=name)
        if 'ProvisioningArtifactSummaries' in rep:
            rret['ProductId'] = rep['ProductViewDetail']['ProductViewSummary']['ProductId']
            lpaths = sc_client.list_launch_paths(
                AcceptLanguage='en', ProductId=rep['ProductViewDetail']['ProductViewSummary']['ProductId'])['LaunchPathSummaries']
            for pa in rep['ProvisioningArtifactSummaries']:
                resp2 = sc_client.describe_provisioning_artifact(
                    AcceptLanguage='en', ProvisioningArtifactId=pa['Id'], ProductId=rep['ProductViewDetail']['ProductViewSummary']['ProductId'], Verbose=True)
                rret['template'] = resp2['Info']['CloudFormationTemplate']
                if resp2['ProvisioningArtifactDetail']['Active']:
                    dinfo = {'ProvisioningArtifactId': resp2['ProvisioningArtifactDetail']
                             ['Id'], 'Active': resp2['ProvisioningArtifactDetail']['Active']}
                    rret['ProvisioningArtifactId'] = resp2['ProvisioningArtifactDetail']['Id']

            rret['pathId'] = lpaths[0]['Id']

    except Exception as e:
        print(e)
    return rret


def sc_find_product(name):
    _ret = 'not found'
    try:
        rep = sc_client.describe_product_as_admin(
            AcceptLanguage='en', Name=name)

        _ret = 'found'
    except Exception as e:
        e = str(e)
        print(e)
        if e.find('ResourceNotFoundException') > -1:
            print(name + ' not found')
    return _ret


def sc_describe_product(name):
    rep = {}
    try:
        rep = sc_client.describe_product_as_admin(
            AcceptLanguage='en', Name=name)
    except Exception as e:
        e = str(e)
        if e.find('ResourceNotFoundException') > -1:
            print(name + ' not found')
            _ret = 'not found'
            return _ret
    ProductId = rep['ProductViewDetail']['ProductViewSummary']['ProductId']
    _ret = []
    for _prodversion in rep['ProvisioningArtifactSummaries']:
        _version = {'ProductId': ProductId, 'ProvisioningArtifactId': _prodversion[
            'Id'], 'Name': _prodversion['Name'], 'Description': _prodversion['Description']}
        _ret.append(_version)
    return _ret


def mgt_ent_sc_updateProduct(_sc_productname, template):
    _prodinfo = sc_describe_product(_sc_productname)
    _versions = len(_prodinfo) + 1
    ProductId = _prodinfo[0]['ProductId']
    version = 'Latest %s.0' % _versions
    description = 'Decription %s' % version
    pa_rep = sc_create_pa(
        ProductId, version, description, template)
    _ret = {'ProductId': ProductId, 'version': version,
            'ProvisioningArtifactId': pa_rep['ProvisioningArtifactDetail']['Id']}
    _prodinfo = sc_describe_product(_sc_productname)
    sc_deactivat_ver(version, _prodinfo)
    return _ret


def bucket_putpriv(bucket, key, body, type):
    srep = s3client.put_object(
        ACL='private', Body=body, Bucket=bucket, Key=key, ContentType=type,)
    print(srep)
    return srep


def bucket_get_obj(bucket, bkey):
    object = s3client.get_object(Bucket=bucket, Key=bkey)
    return(object['Body'].read().decode('utf-8'))


def listdirectry(bucket, prefix):
    _ret = []
    ret = s3client.list_objects_v2(
        Bucket=bucket, Delimiter=',', MaxKeys=123, Prefix=prefix)
    for key in ret['Contents']:
        keyarray = key['Key'].split('/')
        _scount = len(keyarray)
        _ret.append(keyarray[_scount - 2])
        _rret = list(set(_ret))
        _rret.sort()
    _dircount = []
    _ret2 = []
    for _dir in _rret:
        _ncount = 0
        _prefix = f'{prefix}{_dir}/'
        print(_prefix)
        _nret = s3client.list_objects_v2(
            Bucket=bucket, Delimiter=',', MaxKeys=123, Prefix=_prefix)
        if 'Contents' in _nret:
            _ncount = len(_nret['Contents'])
        _entry = f'{_dir}[{_ncount}]'
        _ret2.append(_entry)

    return _ret2


def listdirectryitems(bucket, prefix, strip):
    _ret = []
    ret = s3client.list_objects_v2(
        Bucket=bucket, Delimiter=',', MaxKeys=123, Prefix=prefix)
    if 'Contents' in ret:
        for key in ret['Contents']:
            if strip == 'yes':
                _keyarr = key['Key'].split('/')
                _klen = len(_keyarr)
                _ret.append(_keyarr[_klen - 1])
            else:
                _ret.append(key['Key'])
            _ret.sort()

    return _ret


def prep_function(event):
    bucket = os.environ['S3bucket']
    scproductname = os.environ['scproductname']
    FuncEnhAccFacCFTdeploy = os.environ['FuncEnhAccFacCFTdeploy']
    EnhanceAFfunctionRole = os.environ['EnhanceAFfunctionRole']
    FuncEnhAccFacCFTcheck = os.environ['FuncEnhAccFacCFTcheck']
    FuncEnhAccFacCFTupdate = os.environ['FuncEnhAccFacCFTupdate']
    RoleStepFunction = os.environ['RoleStepFunction']
    prefix = os.environ['prefix']
    stepfunkey = f"{prefix}scenhanceaf_stepfunction.json"
    stepfun = bucket_get_obj(bucket, stepfunkey)
    stepfun = stepfun.replace(
        '__CheckStack__', FuncEnhAccFacCFTcheck)
    stepfun = stepfun.replace(
        '__DeployStack__', FuncEnhAccFacCFTdeploy)
    stepfun = stepfun.replace(
        '__Logger__', FuncEnhAccFacCFTupdate)
    stepfun = stepfun.replace(
        '__UpdateLog__', FuncEnhAccFacCFTupdate)
    stepfun = stepfun.replace(
        '__RoleArn__', RoleStepFunction)
    stepfunkeyout = f"{prefix}scenhanceaf_stepfunction_new.json"

    bucket_putpriv(bucket, stepfunkeyout,
                   stepfun, 'text/json')
    _ret = f'https://{bucket}.s3.amazonaws.com/{stepfunkeyout}'

    return _ret


def get_ous_fromtemplate(template):
    _tp1 = template.split("ManagedOrganizationalUnit:")[
        1].split('Resources:')[0].split(' - ')
    tpar = []
    for gval in _tp1:
        if gval.find('9') > -1:
            t = gval.replace('\n', '')
            tpar.append(t.rstrip())
    return tpar


def update_bucket_notification(event):
    NotificationConfiguration = {
        "LambdaFunctionConfigurations":
        [
            {
                'Id': 'scautoupdate',
                "Events":
                [
                    "s3:ObjectCreated:Put",
                    "s3:ObjectRemoved:Delete",
                    "s3:ObjectRemoved:DeleteMarkerCreated"
                ],
                "Filter":
                {
                    "Key":
                    {
                        "FilterRules":
                        [
                            {
                                "Name": "Prefix",
                                "Value": "content/scenhanceaf/src/postaccount/"
                            },
                            {
                                "Name": "Suffix",
                                "Value": ""
                            }
                        ]
                    }
                },
                "LambdaFunctionArn": event['LambdaEnhanceSCprodUpdate']
            }
        ]
    }
    resp = s3client.put_bucket_notification_configuration(
        Bucket=event['bucket'], NotificationConfiguration=NotificationConfiguration, SkipDestinationValidation=True)
    return(resp)


def ListAccounts():
    _ar = []
    try:
        _ret = org_client.list_roots()
        _id = _ret['Roots'][0]['Id']
        _ret2 = org_client.list_organizational_units_for_parent(
            ParentId=_id)
        for _o in _ret2['OrganizationalUnits']:
            _accts = org_client.list_accounts_for_parent(
                ParentId=_o['Id'])
            for _act in _accts['Accounts']:
                if _o['Name'].find('Core') > -1:
                    donothigpartofcore = 1
                else:
                    _b = f"{_o['Name']}_{_act['Name']}_{_act['Id']}_{_act['Status']}"
                    _ar.append(_b)
            _ar.sort()
    except Exception as e:
        _ar.append('Error_gettingAccounts ' + str(e))
    return _ar


def lambda_handler(event, context):
    #
    # return sc_get_prod_details(os.environ['scproductname'])
    print(json.dumps(event))

    timer = threading.Timer((context.get_remaining_time_in_millis(
    ) / 1000.00) - 0.5, timeout, args=[event, context])
    timer.start()
    status = 'SUCCESS'
    ret = {}

    #
    af_c = 'port-ryugffhloxoss'
    bucket = os.environ['S3bucket']
    scproductname = os.environ['scproductname']
    LambdaEnhanceSCprodUpdate = os.environ['LambdaEnhanceSCprodUpdate']

    prefix = os.environ['prefix']
    sc_product_scenhanceafjson_key = f'{prefix}sc_product_scenhanceaf.json'
    sc_product_scenhanceafjson_key_new = f'{prefix}sc_product_scenhanceaf_new.json'
    sc_profjson_key_new = f'{prefix}sc_product_scenhanceaf_acftnew.json'
    #

    try:
        if event['RequestType'] == 'Create':
            if event['ResourceProperties']['action'] == 'new':
                if event['LogicalResourceId'] == "LambdaEnhanceAFRun":
                    notevent = {
                        'bucket': bucket, 'LambdaEnhanceSCprodUpdate': LambdaEnhanceSCprodUpdate}

                    ret2 = update_bucket_notification(
                        notevent)
                    # return notevent
                configparakey = f'{prefix}src/config/enparameter.json'

                configparajson = json.loads(bucket_get_obj(
                    bucket, configparakey))['ehancedparameters']
                postprefix = f'{prefix}src/postaccount/'
                post_stacks = listdirectryitems(
                    bucket, postprefix, 'yes')
                post_stacks.append('None')
                _prodinfo = sc_get_prod_details(
                    scproductname)
                pou = get_ous_fromtemplate(
                    _prodinfo['template'])
                sc_product_json = json.loads(bucket_get_obj(
                    bucket, sc_product_scenhanceafjson_key))

                sc_product_json['Resources']['Scaccountfactorcustom'][
                    'Properties']['ProductId'] = _prodinfo['ProductId']
                sc_product_json['Resources']['CallStepFunction']['Properties'][
                    'ServiceToken'] = os.environ['EnhanceAFfunction']

                sc_product_json['Parameters']['ManagedOrganizationalUnit']['AllowedValues'] = pou
                originalpara = sc_product_json['Parameters']
                for _para in configparajson:
                    pp = {_para['Name']: {'Description': _para['Description'],
                                          'Type': _para['Type'], 'AllowedValues': post_stacks}}
                    sc_product_json['Parameters'][_para['Name']] = {
                        'Description': _para['Description'], 'Type': _para['Type'], 'AllowedValues': post_stacks, 'Default': 'None'}
                    sc_product_json['Resources']['CallStepFunction']['Properties'][_para['Name']] = {
                        "Ref": _para['Name']}
                if 'StepFunctionEnhanceAF' in event['ResourceProperties']:
                    sc_product_json['Resources']['CallStepFunction']['Properties'][
                        'sfnarn'] = event['ResourceProperties']['StepFunctionEnhanceAF']

                bucket_putpriv(bucket, sc_product_scenhanceafjson_key_new, json.dumps(
                    sc_product_json, indent=4), 'text/json')
                bucket_putpriv(
                    bucket, sc_profjson_key_new, _prodinfo['template'], 'text/json')
                ret['loadtemplatefromurl'] = f'https://{bucket}.s3.amazonaws.com/{sc_product_scenhanceafjson_key_new}'
                ret['stepfunctionurl'] = prep_function(
                    event)

                # SC product templates only
                sc_product_scenhanceafjson_Tkey_new = f'{prefix}sc_product_scenhanceaf_templates_new.json'
                sc_product_scenhanceafjson_Tkey = f'{prefix}sc_product_scenhanceaf_templates.json'
                targetAccounts = ListAccounts()
                sc_product_jsonTeplate = json.loads(
                    bucket_get_obj(bucket, sc_product_scenhanceafjson_Tkey))
                sc_product_jsonTeplate['Resources']['CallStepFunction'][
                    'Properties']['ServiceToken'] = os.environ['EnhanceAFfunction']
                sc_product_jsonTeplate['Parameters']['TargetAccount']['AllowedValues'] = targetAccounts
                for _para in configparajson:
                    pp = {_para['Name']: {'Description': _para['Description'],
                                          'Type': _para['Type'], 'AllowedValues': post_stacks}}
                    sc_product_jsonTeplate['Parameters'][_para['Name']] = {
                        'Description': _para['Description'], 'Type': _para['Type'], 'AllowedValues': post_stacks, 'Default': 'None'}
                    sc_product_jsonTeplate['Resources']['CallStepFunction']['Properties'][_para['Name']] = {
                        "Ref": _para['Name']}
                if 'StepFunctionEnhanceAF' in event['ResourceProperties']:
                    sc_product_jsonTeplate['Resources']['CallStepFunction']['Properties'][
                        'sfnarn'] = event['ResourceProperties']['StepFunctionEnhanceAF']

                bucket_putpriv(bucket, sc_product_scenhanceafjson_Tkey_new, json.dumps(
                    sc_product_jsonTeplate, indent=4), 'text/json')
                ret['loadtemplatefromurltmplate'] = f'https://{bucket}.s3.amazonaws.com/{sc_product_scenhanceafjson_Tkey_new}'

        if event['RequestType'] == 'Delete':
            ret['state'] = 'Deleting'

    except Exception as e:
        logging.error('Exception: %s' % e, exc_info=True)
        status = 'FAILED'
    finally:
        timer.cancel()
        print(json.dumps(ret))
        cfnresponse(event, context, status, ret)
        return ret
