import json
import logging
import re
import random
import time
import boto3

from botocore.exceptions import ClientError
_rand = str(random.randint(10, 2400))
#
iam_client = boto3.client('iam')


logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)


def assume_role(account):
    assumedRoleObject = {}
    sts_client = boto3.client('sts')
    role_arn = 'arn:aws:iam::' + \
        account['AccountId'] + ':role/' + \
        account['adminRole']
    try:
        assumedRoleObject = sts_client.assume_role(
            RoleArn=role_arn,
            RoleSessionName="NewAccountRole"
        )
    except Exception as e:
        logger.warning(e)
        assumedRoleObject['status'] = str(e)

    return assumedRoleObject


def check_status_cloudformation(event):

    logger.debug("Checking Resources")
    stack_name = event['stack_name']
    stack_region = event['stack_region']
    TemplateURL = event['TemplateURL']
    taraccount = event['taraccount']
    stack_status = ''
    #
    # Get target account info
    targetAcct = {'AccountId': taraccount,
                  'adminRole': 'AWSControlTowerExecution'}
    acct_b = assume_role(targetAcct)

    client = boto3.client('cloudformation',
                          aws_access_key_id=acct_b['Credentials']['AccessKeyId'],
                          aws_secret_access_key=acct_b['Credentials']['SecretAccessKey'],
                          aws_session_token=acct_b['Credentials']['SessionToken'],
                          region_name=stack_region)

    logger.info("Checking  stack " +
                stack_name + " in " + taraccount)
    logger.info(
        "Checking stack TemplateURL[" + TemplateURL + "]")
    #
    event['stack_status'] = client.describe_stacks(
        StackName=stack_name)['Stacks'][0]['StackStatus']
    return event


def lambda_handler(event, context):
    print(json.dumps(event))
    event = check_status_cloudformation(event)
    return event
